==> code_id_storage-0.1.2.tar.gz: lib/code_services/storage/storage.dart <==
import 'dart:async';

import 'package:code_id_storage/code_interfaces/code_interfaces.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:hive_flutter/hive_flutter.dart';

class StorageImpl<T> implements IStorage<T> {
  Box<T>? _box;
  LazyBox<T>? _lazyBox;

  late HiveInterface _hive;
  late FlutterSecureStorage _ss;

  StorageImpl() {
    _hive = Hive;
  }

  ///Register [Hive] List of [TypeAdapter] and init the [Hive] flutter
  ///
  /// Tries to register [adapter] to registry. If another adapter with same
  /// typeId had been already registered an exception will thrown or the adapter
  /// will be overridden if [override] set to `true`. Please note that internal
  /// adapters are registered and maintained by hive itself. Use [internal]
  /// parameter only if you want to override exists adapter implementation.
  @override
  Future<void> init({
    String? subdir,
  }) async {
    await _hive.initFlutter(subdir);
    _ss = const FlutterSecureStorage();
  }

  @override
  void registerAdapter(
    T adapter, {
    bool internal = false,
    bool override = false,
  }) {
    _hive.registerAdapter(
      adapter as TypeAdapter<T>,
      internal: internal,
      override: override,
    );
  }

  @override
  void registerAdapterManual({required Function(HiveInterface) adapter}) {
    adapter.call(_hive);
  }

  ///Open [Hive] box and get the data into memory
  @override
  Future<void> openBox(
    String boxName, {
    bool useEncryption = true,
  }) async {
    if (_box == null && !_hive.isBoxOpen('boxName')) {
      final List<int> hiveKey = await _hiveKeys();

      _box = await _hive.openBox<T>(
        boxName,
        encryptionCipher: useEncryption ? HiveAesCipher(hiveKey) : null,
        crashRecovery: true,
      );
      debugPrint('====================Open Box=================');
      debugPrint('Database $boxName Open');
      debugPrint('=============================================');
    } else {
      debugPrint('====================Open Box=================');
      debugPrint('Database $boxName is Already Open');
      debugPrint('=============================================');
    }
  }

  ///Open [Hive] box and get data without inset it into memory
  /// !Do not open lazy box with box on same function or the performance will become bad
  @override
  Future<void> openLazyBox(
    String boxName, {
    bool useEncryption = true,
  }) async {
    if (_lazyBox == null && !_hive.isBoxOpen('boxName')) {
      final List<int> hiveKey = await _hiveKeys();

      _lazyBox = await _hive.openLazyBox<T>(
        boxName,
        encryptionCipher: useEncryption ? HiveAesCipher(hiveKey) : null,
        crashRecovery: true,
      );
      debugPrint('====================Open Box=================');
      debugPrint('Database $boxName Open');
      debugPrint('=============================================');
    } else {
      debugPrint('====================Open Box=================');
      debugPrint('Database $boxName is Already Open');
      debugPrint('=============================================');
    }
  }

  @override
  Future close() async {
    await _box?.close();
    await _lazyBox?.close();
    return;
  }

  @override
  Future<void> putData({
    required Map<dynamic, T> data,
    bool autoClose = false,
  }) async {
    try {
      debugPrint('====================Put Data=================');
      debugPrint('$data');
      debugPrint('=============================================');
      if (_box != null) {
        await _box!.putAll(data);
      }
      if (_lazyBox != null) {
        await _lazyBox!.putAll(data);
      }
      debugPrint('================Put Data Success=============');
    } catch (e) {
      debugPrint('================Put Data Failed==============');
      debugPrint(e.toString());
      debugPrint('=============================================');
    } finally {
      if (autoClose) {
        close();
      }
    }
    return;
  }

  @override
  Future<void> putDatum({
    required dynamic key,
    required T value,
    bool autoClose = false,
  }) async {
    try {
      debugPrint('====================Put Data=================');
      debugPrint('$value');
      debugPrint('=============================================');
      if (_box != null) {
        await _box!.put(key, value);
      }
      if (_lazyBox != null) {
        await _lazyBox!.put(key, value);
      }
      debugPrint('================Put Data Success=============');
    } catch (e) {
      debugPrint('================Put Data Failed==============');
      debugPrint(e.toString());
      debugPrint('=============================================');
    } finally {
      if (autoClose) {
        close();
      }
    }
    return;
  }

  @override
  Future<void> putListData({
    required Iterable<T> dataList,
    bool autoClose = false,
  }) async {
    try {
      debugPrint('====================Put Data=================');
      int x = 0;
      await Future.wait(dataList.map((val) async {
        if (_box != null) {
          await _box!.add(val);
        }
        if (_lazyBox != null) {
          await _lazyBox!.add(val);
        }
        x++;
        debugPrint('key $x : $val');
      }));

      debugPrint('=============================================');
    } catch (e) {
      debugPrint('================Put Data Failed==============');
      debugPrint(e.toString());
      debugPrint('=============================================');
    } finally {
      if (autoClose) {
        close();
      }
    }

    return;
  }

  @override
  FutureOr<T?> getData({
    required dynamic key,
    bool autoClose = false,
  }) async {
    try {
      debugPrint('====================Get Data=================');
      T? value;
      if (_box != null && _box!.isOpen) {
        value = _box!.get(key);
      }
      if (_lazyBox != null && _lazyBox!.isOpen) {
        value = await _lazyBox!.get(key);
      }
      debugPrint('$value');
      debugPrint('=============================================');
      return value;
    } catch (e) {
      debugPrint('================Get Data Failed==============');
      debugPrint(e.toString());
      debugPrint('=============================================');
      rethrow;
    } finally {
      if (autoClose) {
        close();
      }
    }
  }

  // @override
  // Future<T?> getLazyData({required String key}) async {
  //   debugPrint('====================Get Data=================');
  //   final T? value = await _lazyBox!.get(key);
  //   debugPrint('$value');
  //   debugPrint('=============================================');
  //   return value;
  // }

  @override
  IList<T>? getListData({
    bool autoClose = false,
  }) {
    try {
      IList<T> data = <T>[].lock;
      for (int x = 0; x < _box!.length; x++) {
        final T? item = _box!.getAt(x);
        if (item != null) {
          data = data.add(item);
        }
      }

      return data;
    } catch (e) {
      rethrow;
    } finally {
      if (autoClose) {
        close();
      }
    }
  }

  @override
  Future<IList<T>?> getLazyListData({
    bool autoClose = false,
  }) async {
    try {
      IList<T> data = <T>[].lock;
      for (int x = 0; x < _lazyBox!.length; x++) {
        final T? item = await _lazyBox!.getAt(x);
        if (item != null) {
          data = data.add(item);
        }
      }

      return data;
    } catch (e) {
      rethrow;
    } finally {
      if (autoClose) {
        close();
      }
    }
  }

  @override
  Future<void> clear() async {
    if (_box != null) {
      await _box!.clear();
    }
    if (_lazyBox != null) {
      await _lazyBox!.clear();
    }
    return;
  }

  @override
  Future<void> delete({
    required String key,
    bool autoClose = false,
  }) async {
    try {
      if (_box != null) {
        await _box!.delete(key);
      }
      if (_lazyBox != null) {
        await _lazyBox!.delete(key);
      }
      return;
    } catch (e) {
      return;
    } finally {
      if (autoClose) {
        close();
      }
    }
  }

  @override
  Future<void> deleteList({
    required Iterable<dynamic> key,
    bool autoClose = false,
  }) async {
    try {
      if (_box != null) {
        await _box!.deleteAll(key);
      }
      if (_lazyBox != null) {
        await _lazyBox!.deleteAll(key);
      }

      return;
    } catch (e) {
      return;
    } finally {
      if (autoClose) {
        close();
      }
    }
  }

  Future<List<int>> _hiveKeys() async {
    // const FlutterSecureStorage ss = FlutterSecureStorage();

    const AndroidOptions aOptions = AndroidOptions(
      encryptedSharedPreferences: true,
    );

    const IOSOptions iOptions = IOSOptions(
      accessibility: KeychainAccessibility.first_unlock,
    );
    String? stringKey = await _ss.read(
      key: 'boxKey',
      aOptions: aOptions,
      iOptions: iOptions,
    );
    late List<int> hiveKey;
    if (stringKey != null) {
      hiveKey = stringKey.codeUnits;
    } else {
      hiveKey = Hive.generateSecureKey();
      final Uint8List bytes = Uint8List.fromList(hiveKey);
      stringKey = String.fromCharCodes(bytes);
      // if (Platform.isAndroid) {
      await _ss.write(
        key: 'boxKey',
        value: stringKey,
        aOptions: aOptions,
        iOptions: iOptions,
      );
      // } else {
      //   await _ss.write(key: 'boxKey', value: stringKey, iOptions: iOptions);
      // }
    }
    return hiveKey;
  }

  @override
  bool isOpen() {
    if (_box != null) {
      return _box!.isOpen;
    }
    if (_lazyBox != null) {
      return _lazyBox!.isOpen;
    }
    return false;
  }

  @override
  ValueListenable<Box<T>> listenable({
    List? keys,
  }) {
    return _box!.listenable(
      keys: keys,
    );
  }

  @override
  ValueListenable<LazyBox<T>> lazyListenable({
    List? keys,
  }) {
    return _lazyBox!.listenable(
      keys: keys,
    );
  }
}

//global const to access storage
// ignore: non_constant_identifier_names
final IStorage Storage = StorageImpl();

==> code_id_storage-0.1.3.tar.gz: lib/code_services/storage/storage.dart <==
import 'dart:async';

import 'package:code_id_storage/code_interfaces/code_interfaces.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:hive_flutter/hive_flutter.dart';

class StorageImpl<T> implements IStorage<T> {
  Box<T>? _box;
  LazyBox<T>? _lazyBox;

  late HiveInterface _hive;
  late FlutterSecureStorage _ss;

  StorageImpl() {
    _hive = Hive;
  }

  ///Register [Hive] List of [TypeAdapter] and init the [Hive] flutter
  ///
  /// Tries to register [adapter] to registry. If another adapter with same
  /// typeId had been already registered an exception will thrown or the adapter
  /// will be overridden if [override] set to `true`. Please note that internal
  /// adapters are registered and maintained by hive itself. Use [internal]
  /// parameter only if you want to override exists adapter implementation.
  @override
  Future<void> init({
    String? subdir,
  }) async {
    await _hive.initFlutter(subdir);
    _ss = const FlutterSecureStorage();
  }

  @override
  void registerAdapter<S>(
    TypeAdapter<S> adapter, {
    bool internal = false,
    bool override = false,
  }) {
    _hive.registerAdapter<S>(
      adapter,
      internal: internal,
      override: override,
    );
  }

  @override
  void registerAdapterManual({required Function(HiveInterface) adapter}) {
    adapter.call(_hive);
  }

  ///Open [Hive] box and get the data into memory
  @override
  Future<void> openBox(
    String boxName, {
    bool useEncryption = true,
  }) async {
    if (_box == null && !_hive.isBoxOpen('boxName')) {
      final List<int> hiveKey = await _hiveKeys();

      _box = await _hive.openBox<T>(
        boxName,
        encryptionCipher: useEncryption ? HiveAesCipher(hiveKey) : null,
        crashRecovery: true,
      );
      debugPrint('====================Open Box=================');
      debugPrint('Database $boxName Open');
      debugPrint('=============================================');
    } else {
      debugPrint('====================Open Box=================');
      debugPrint('Database $boxName is Already Open');
      debugPrint('=============================================');
    }
  }

  ///Open [Hive] box and get data without inset it into memory
  /// !Do not open lazy box with box on same function or the performance will become bad
  @override
  Future<void> openLazyBox(
    String boxName, {
    bool useEncryption = true,
  }) async {
    if (_lazyBox == null && !_hive.isBoxOpen('boxName')) {
      final List<int> hiveKey = await _hiveKeys();

      _lazyBox = await _hive.openLazyBox<T>(
        boxName,
        encryptionCipher: useEncryption ? HiveAesCipher(hiveKey) : null,
        crashRecovery: true,
      );
      debugPrint('====================Open Box=================');
      debugPrint('Database $boxName Open');
      debugPrint('=============================================');
    } else {
      debugPrint('====================Open Box=================');
      debugPrint('Database $boxName is Already Open');
      debugPrint('=============================================');
    }
  }

  @override
  Future close() async {
    await _box?.close();
    await _lazyBox?.close();
    return;
  }

  @override
  Future<void> putData({
    required Map<dynamic, T> data,
    bool autoClose = false,
  }) async {
    try {
      debugPrint('====================Put Data=================');
      debugPrint('$data');
      debugPrint('=============================================');
      if (_box != null) {
        await _box!.putAll(data);
      }
      if (_lazyBox != null) {
        await _lazyBox!.putAll(data);
      }
      debugPrint('================Put Data Success=============');
    } catch (e) {
      debugPrint('================Put Data Failed==============');
      debugPrint(e.toString());
      debugPrint('=============================================');
    } finally {
      if (autoClose) {
        close();
      }
    }
    return;
  }

  @override
  Future<void> putDatum({
    required dynamic key,
    required T value,
    bool autoClose = false,
  }) async {
    try {
      debugPrint('====================Put Data=================');
      debugPrint('$value');
      debugPrint('=============================================');
      if (_box != null) {
        await _box!.put(key, value);
      }
      if (_lazyBox != null) {
        await _lazyBox!.put(key, value);
      }
      debugPrint('================Put Data Success=============');
    } catch (e) {
      debugPrint('================Put Data Failed==============');
      debugPrint(e.toString());
      debugPrint('=============================================');
    } finally {
      if (autoClose) {
        close();
      }
    }
    return;
  }

  @override
  Future<void> putListData({
    required Iterable<T> dataList,
    bool autoClose = false,
  }) async {
    try {
      debugPrint('====================Put Data=================');
      int x = 0;
      await Future.wait(dataList.map((val) async {
        if (_box != null) {
          await _box!.add(val);
        }
        if (_lazyBox != null) {
          await _lazyBox!.add(val);
        }
        x++;
        debugPrint('key $x : $val');
      }));

      debugPrint('=============================================');
    } catch (e) {
      debugPrint('================Put Data Failed==============');
      debugPrint(e.toString());
      debugPrint('=============================================');
    } finally {
      if (autoClose) {
        close();
      }
    }

    return;
  }

  @override
  FutureOr<T?> getData({
    required dynamic key,
    bool autoClose = false,
  }) async {
    try {
      debugPrint('====================Get Data=================');
      T? value;
      if (_box != null && _box!.isOpen) {
        value = _box!.get(key);
      }
      if (_lazyBox != null && _lazyBox!.isOpen) {
        value = await _lazyBox!.get(key);
      }
      debugPrint('$value');
      debugPrint('=============================================');
      return value;
    } catch (e) {
      debugPrint('================Get Data Failed==============');
      debugPrint(e.toString());
      debugPrint('=============================================');
      rethrow;
    } finally {
      if (autoClose) {
        close();
      }
    }
  }

  // @override
  // Future<T?> getLazyData({required String key}) async {
  //   debugPrint('====================Get Data=================');
  //   final T? value = await _lazyBox!.get(key);
  //   debugPrint('$value');
  //   debugPrint('=============================================');
  //   return value;
  // }

  @override
  IList<T>? getListData({
    bool autoClose = false,
  }) {
    try {
      IList<T> data = <T>[].lock;
      for (int x = 0; x < _box!.length; x++) {
        final T? item = _box!.getAt(x);
        if (item != null) {
          data = data.add(item);
        }
      }

      return data;
    } catch (e) {
      rethrow;
    } finally {
      if (autoClose) {
        close();
      }
    }
  }

  @override
  Future<IList<T>?> getLazyListData({
    bool autoClose = false,
  }) async {
    try {
      IList<T> data = <T>[].lock;
      for (int x = 0; x < _lazyBox!.length; x++) {
        final T? item = await _lazyBox!.getAt(x);
        if (item != null) {
          data = data.add(item);
        }
      }

      return data;
    } catch (e) {
      rethrow;
    } finally {
      if (autoClose) {
        close();
      }
    }
  }

  @override
  Future<void> clear() async {
    if (_box != null) {
      await _box!.clear();
    }
    if (_lazyBox != null) {
      await _lazyBox!.clear();
    }
    return;
  }

  @override
  Future<void> delete({
    required String key,
    bool autoClose = false,
  }) async {
    try {
      if (_box != null) {
        await _box!.delete(key);
      }
      if (_lazyBox != null) {
        await _lazyBox!.delete(key);
      }
      return;
    } catch (e) {
      return;
    } finally {
      if (autoClose) {
        close();
      }
    }
  }

  @override
  Future<void> deleteList({
    required Iterable<dynamic> key,
    bool autoClose = false,
  }) async {
    try {
      if (_box != null) {
        await _box!.deleteAll(key);
      }
      if (_lazyBox != null) {
        await _lazyBox!.deleteAll(key);
      }

      return;
    } catch (e) {
      return;
    } finally {
      if (autoClose) {
        close();
      }
    }
  }

  Future<List<int>> _hiveKeys() async {
    // const FlutterSecureStorage ss = FlutterSecureStorage();

    const AndroidOptions aOptions = AndroidOptions(
      encryptedSharedPreferences: true,
    );

    const IOSOptions iOptions = IOSOptions(
      accessibility: KeychainAccessibility.first_unlock,
    );
    String? stringKey = await _ss.read(
      key: 'boxKey',
      aOptions: aOptions,
      iOptions: iOptions,
    );
    late List<int> hiveKey;
    if (stringKey != null) {
      hiveKey = stringKey.codeUnits;
    } else {
      hiveKey = Hive.generateSecureKey();
      final Uint8List bytes = Uint8List.fromList(hiveKey);
      stringKey = String.fromCharCodes(bytes);
      // if (Platform.isAndroid) {
      await _ss.write(
        key: 'boxKey',
        value: stringKey,
        aOptions: aOptions,
        iOptions: iOptions,
      );
      // } else {
      //   await _ss.write(key: 'boxKey', value: stringKey, iOptions: iOptions);
      // }
    }
    return hiveKey;
  }

  @override
  bool isOpen() {
    if (_box != null) {
      return _box!.isOpen;
    }
    if (_lazyBox != null) {
      return _lazyBox!.isOpen;
    }
    return false;
  }

  @override
  ValueListenable<Box<T>> listenable({
    List? keys,
  }) {
    return _box!.listenable(
      keys: keys,
    );
  }

  @override
  ValueListenable<LazyBox<T>> lazyListenable({
    List? keys,
  }) {
    return _lazyBox!.listenable(
      keys: keys,
    );
  }
}

//global const to access storage
// ignore: non_constant_identifier_names
final IStorage Storage = StorageImpl();
